//ICC-AVR application builder : 23.01.2010 22:42:36
// Target : T2313
// Crystal: 7.3728Mhz

#include <iot2313v.h>
#include <macros.h>

//    sflag
#define TX1Complete		   0x01
#define RX1Complete		   0x02


#pragma global_register sflag:20 bcnt:21 divt:22 temp:23


//******************************************************************************	
#define SetRed() \
	PORTB &=~ 0x04;   \
	PORTB |= 0x02;   \

#define SetGreen() \
	PORTB &=~ 0x02;   \
	PORTB |= 0x04;   \

#define GetState() \
	PORTB |= 0x20; \
	PORTB &=~ 0x20; \


#define Transmitter1On() \
	PORTD |= 0x20;   \

#define Transmitter1Off() \
	PORTD &=~ 0x20;   \

#define Transmitter0On() \
	PORTD |= 0x40;   \

#define Transmitter0Off() \
	PORTD &=~ 0x40;   \


#define SendUART1() \
	TCCR0B = 0x02; 	   	   	 // start timer   \
	PORTD &=~ 0x10;   	  	 // set start bit \
	asm("ldi R21,0x0A\n");   // set bit counter \
	sflag &=~ TX1Complete;   // reset TX1Complete flag \
	
//******************************************************************************	


unsigned char sflag, temp, bcnt, tm;

void port_init(void)
{
 PORTA = 0x07;
 DDRA  = 0x00; 

 PORTB = 0xDC;
 DDRB  = 0xA7; 
 
 PORTD = 0x1F;
 DDRD  = 0x72; 
}


// TIMER0 initialize - prescale:8, WGM: CTC
void timer0_init(void)
{
 TCCR0B = 0x00; //stop timer
 TCNT0 = 0xA1; 	//set count
 OCR0A = 0x5F; 	//set count
 TCCR0A = 0x02; //set wavefofm generation mode 2-CTC, TCNT0 cleared on OCOA
}


//       ,    
#pragma interrupt_handler timer0_compa_isr:14
void timer0_compa_isr(void)
{
 temp = USIDR;
 asm("bst R23,0\n");	   // T-flag = bit0 R23
 temp >>= 1;
 USIDR = temp;
 temp = PORTD;
 asm("bld R23,0x04\n");	   // TX1 = T-flag
 if(2==bcnt)
 {
  temp |= 0x10;
 }
 PORTD = temp;
 bcnt--;
// asm("lsr R23\n");		   // shift R23 right
// asm("sts _UDR1,R23\n");   // UDR1 = R23
 if(!bcnt)
 {
// asm("dec R21\n");		   // decrement bit counter
// asm("brne LTCA0\n");	   // if not zero, goto label LTCA0
  asm("out 0x33,R21\n");	   // TCCR0B = 0
  Transmitter1Off();		   // 
  asm("ori R20,0x01\n");	   // set TX1Coplete flag
 }			 
// asm("LTCA0:\n");
}


//UART0 initialize
// desired baud rate: 9600
// actual: baud rate:9600 (0,0%)
void uart0_init(void)
{
 UCSRB = 0x00; //disable while setting baud rate
 UBRRH = 0x00; //set baud rate upper
 UBRRL = 0x2F; //set baud rate lower
 UCSRA = 0x00;
 UCSRC = 0x06;
 UCSRB = 0xD8; //enable
}


#pragma interrupt_handler uart0_rx_isr:8
void uart0_rx_isr(void)
{
/* UDR1 = UDR;
 sflag &=~ TX1Complete;
 SendUART1();*/
}


#pragma interrupt_handler uart0_tx_isr:10
void uart0_tx_isr(void)
{
// Transmitter0Off();
}


#pragma interrupt_handler int1_isr:3
void int1_isr(void)
{
 if(sflag & TX1Complete)  	//    
 {
  GIMSK = 0;				//   INT1
  TCNT0 = 0xD2;				// set count
//  TCCR0A = 0x02;			// set WGM mode
  TCCR0B = 0x02;			// start timer
  TIMSK = 0x82;				//      0
  bcnt = 9;					//     
 }// if(sflag & TX1Complete)
}


#pragma interrupt_handler timer0_ovf_isr:7
void timer0_ovf_isr(void)
{
 TCNT0 = 0xA4; 			  		//reload counter value
// asm("lds R23,_UDR1\n");
 asm("sec\n");
 asm("sbis 0x10,3\n");
 asm("clc\n");
 asm("ror R23\n");
// asm("sts _UDR1,R23\n");
 asm("dec R21\n");
 asm("brne LTO0\n");

 
 asm("out 0x33,R21\n");	   		// TCCR0B = 0
 
 sflag |= RX1Complete;
 
// asm("ori R20,0x02\n");			// set RX1Coplete flag	

/* 
 TCCR0B = 0x00; //stop timer
 OCR0A = 0x5F; 		 			//set count
 OCR0B = 0x04; 					//set count
 TCCR0A = 0x02; 				//start timer

 GIMSK = 0x80;		
 TIMSK = 80; 
*/
 asm("LTO0:\n");
}


//TIMER1 initialize - prescale:1024
// WGM: 0) Normal, TOP=0xFFFF
// desired value: 1Sec
// actual value:  1,000Sec (0,0%)
void timer1_init(void)
{
 TCCR1B = 0x00; //stop timer
 TCNT1H = 0xE3; //set count value
 TCNT1L = 0xE1;
 TCCR1A = 0x00;
 TCCR1B = 0x05; //start Timer
}


#pragma interrupt_handler timer1_ovf_isr:6
void timer1_ovf_isr(void)
{
 //TIMER1 has overflowed
 TCNT1H = 0xE3; //reload counter high value
 TCNT1L = 0xE1; //reload counter low value

 Transmitter1On();			
 USIDR = tm;
 SendUART1();
}


//call this routine to initialize all peripherals
void init_devices(void)
{
 //stop errant interrupts until set up
 CLI(); //disable all interrupts
 port_init();
 timer0_init();
 timer1_init();
 uart0_init();

 MCUCR = 0x08;
 GIMSK = 0x80;
 TIMSK = 0x85;
 SEI(); //re-enable interrupts
 //all peripherals are now initialized
}


void GetStatus(void)
{
 GetState();
}


void main(void)
{
 unsigned int i; 

 init_devices();
 sflag = 0x01;
 
// tm = 0x55;
// SetGreen();

 
 Transmitter0On();
 UDR = 0x57;

 
// GetStatus();
 
  
 Transmitter1On();		
 USIDR = 0x57;
// UDR1 = 0xAA;
 for(i=74;i>0;i--);
 SendUART1();

 do
 {
/*
  if(sflag & RX1Complete)
  {
     
   sflag &=~ RX1Complete;
   if(0x11 == USIDR) tm++;
   SetRed();

// CLI(); //disable all interrupts
 TCCR0B = 0x00; //stop timer
 TCNT0 = 0x00; 	//set count
 OCR0A = 0x5F; 	//set count
 OCR0B = 0x04; 	//set count
 TCCR0A = 0x02; //start timer

 MCUCR = 0x08;
 GIMSK = 0x80;
 TIMSK = 0x85;
// SEI(); //re-enable interrupts
 //all peripherals are now initialized
   
  
//   Transmitter1On();
//   UDR1 = tm;
  }
  */
 }while(1);
}