//ICC-AVR application builder : 14.02.2010 1:39:47
// Target : T2313
// Crystal: 7.3728Mhz

#include <iot2313v.h>
#include <macros.h>

#pragma global_register sflag:20 bcnt:21 UDR1:22 tm:23 

//    sflag
#define TX1Complete		   0x01
#define RX1Complete		   0x02

//******************************************************************************	

#define SetRed() \
	PORTB &=~ 0x04;   \
	PORTB |= 0x02   \

#define SetGreen() \
	PORTB &=~ 0x02;   \
	PORTB |= 0x04   \


#define Transmitter1On(); \
	PORTD |= 0x20   	 	\

#define Transmitter1Off() \
	PORTD &=~ 0x20   \

#define Transmitter0On() \
	PORTD |= 0x40   \

#define Transmitter0Off() \
	PORTD &=~ 0x40   \
	
#define StartUART1() \
	TCCR0B = 0x02;   \
	PORTD &=~ 0x10;  \
	bcnt = 0x0A;   \
	sflag &=~ TX1Complete   \

//******************************************************************************	



unsigned char sflag, bcnt, UDR1, tm;

void port_init(void)
{
 PORTA = 0x07;
 DDRA  = 0x00; 

 PORTB = 0xDA;
 DDRB  = 0xE7; 

 PORTD = 0x1F;
 DDRD  = 0x72; 
}

//TIMER0 initialize - prescale:8
// WGM: Normal
// desired value: 104uSec
// actual value: 103,082uSec (0,9%)
void timer0_init(void)
{
 TCCR0B = 0x00; //stop timer
 TCNT0 = 0x00; //set count
 OCR0A = 0x5F; //set count
 TCCR0A = 0x02; //start timer
}


#pragma interrupt_handler timer0_ovf_isr:7
void timer0_ovf_isr(void)
{
 asm("ldi R23,0xA1\n"); 		// new value for TCNT0
 asm("out 0x32,R23\n");			// reload counter value

 UDR1 >>= 1;					// shift UDR1 right
 asm("in R23,0x10\n");			// read PIND in tm
 asm("bst R23,3\n");			// T-flag = PD3
 asm("bld R22,7\n");			// bit 7 UDR1 = T-flag
 
 bcnt--;
 
 if(!bcnt) 
 {
  TCCR0B = bcnt;
  TCNT0 = bcnt;
//  GIMSK = 0x80;
  sflag |= RX1Complete;
 }
}


#pragma interrupt_handler timer0_compa_isr:14
void timer0_compa_isr(void)
{
 bcnt--; 				   // decrement byte counter
 if(bcnt)				   // if bcnt not equal 0
 {
  asm("bst R22,0\n");	   // T-flag = bit0 UDR1
  asm("lsr R22\n");	   	   // shift UDR1 right
  tm = PORTD;
  asm("bld R23,0x04\n");   // TX1 pin = T-flag
  if(1 == bcnt)			   // if need send STOP-byte
  {
   tm |= 0x10;			   // set TX1 pin in high
  }// if(2 == bcnt)
  PORTD = tm;			   // write new state into port
 }
 else
 {
  TCCR0B = bcnt;		   // stop timer
  Transmitter1Off();	   // 
  sflag |= TX1Complete;	   //
 }// else
}


//UART0 initialize
// desired baud rate: 9600
// actual: baud rate:9600 (0,0%)
void uart0_init(void)
{
 UCSRB = 0x00; //disable while setting baud rate
 UBRRH = 0x00; //set baud rate upper
 UBRRL = 0x2F; //set baud rate lower
 UCSRA = 0x00;
 UCSRC = 0x06;
 UCSRB = 0xD8; //enable
}


#pragma interrupt_handler uart0_rx_isr:8
void uart0_rx_isr(void)
{
 //uart has received a character in UDR
 NOP();
 Transmitter1Off();
}


#pragma interrupt_handler uart0_tx_isr:10
void uart0_tx_isr(void)
{
 //character has been transmitted
 NOP();
}


#pragma interrupt_handler int1_isr:3
void int1_isr(void)
{
 if(sflag & TX1Complete)  	//    
 {
//  GIMSK = 0;				//   INT1
  TCNT0 = 0xD2;				// set count
  TCCR0B = 0x02;			// start timer
  bcnt = 9;					//     
 }// if(sflag & TX1Complete)
}


//call this routine to initialize all peripherals
void init_devices(void)
{
 //stop errant interrupts until set up
 CLI(); //disable all interrupts
 port_init();
 timer0_init();
 uart0_init();

 MCUCR = 0x08;
 GIMSK = 0x80;
 TIMSK = 0x03;
 SEI(); //re-enable interrupts
 //all peripherals are now initialized
}

void main(void)
{
 unsigned char i;
 
 init_devices();

 SetGreen();
 sflag = 0x01;
 
 Transmitter0On();
 UDR = 0x67;

/* 
 for(i=141;i>0;i--);
 
 Transmitter1On();
 UDR1 = 0x67;
 StartUART1();
*/
 
 do
 {
 }while(1);
}
 